@@ -95,14 +95,16 @@ do { \
 
 static void usage(const char *pname)
 {
-	printf("%s usage:\n\n\t%s [ -h | -V | -n | -r | -s] \n\n", pname, pname);
+	printf("%s usage:\n\n\t%s [ -h | -V | -n | -r | -c | -s | -t secs ]\n\n", pname, pname);
 	printf("\t\t  -h : this help\n");
 	printf("\t\t  -V : print version number\n");
 	printf("\t\t  -n : no ATR analysis\n");
 	printf("\t\t  -r : only lists readers\n");
+	printf("\t\t  -c : only lists cards\n");
 	printf("\t\t  -s : stress mode\n");
 	printf("\t\t  -q : quiet mode\n");
 	printf("\t\t  -v : verbose mode (default)\n");
+	printf("\t\t  -t secs : quit after secs seconds\n");
 	printf("\n");
 }
 
@@ -115,6 +117,22 @@ const char *cub2 = "";
 const char *cub3 = "";
 const char *cpl = "";
 
+time_t start_time;
+
+typedef struct
+{
+	const char *pname;
+	Boolean analyse_atr;
+	Boolean stress_card;
+	Boolean print_version;
+	Boolean verbose;
+	Boolean only_list_readers;
+	Boolean only_list_cards;
+	long maxtime; // in seconds
+} options_t;
+
+static options_t Options;
+
 static Boolean is_member(const char *  item, const char * list[])
 {
 	int i = 0;
@@ -131,7 +149,9 @@ static void initialize_terminal(void)
 		"Eterm", "rxvt", "rxvt-unicode", "cygwin", 0};
 	const char *no_ansi_cursor_terms[] = {"dumb", "emacs", 0};
 	const char *term = getenv("TERM");
-	if (term == 0)
+
+	/* do not use color if stdout is redirected */
+	if (!isatty(fileno(stdout)) || (NULL == term))
 	{
 		term = "dumb";
 	}
@@ -157,6 +177,17 @@ static void initialize_terminal(void)
 }
 /* There should be no \033 beyond this line! */
 
+static Boolean reached_maxtime(void)
+{
+	if (Options.maxtime)
+	{
+		time_t t = time(NULL);
+		if (t - start_time > Options.maxtime)
+			return True;
+	}
+	return False;
+}
+
 Boolean spinning_interrupted = False;
 unsigned int spin_state = 0;
 static void spin_start(void)
@@ -168,6 +199,13 @@ static void spin_update(void)
 {
 	char patterns[] = {'-', '\\', '|', '/'};
 	char c = patterns[spin_state];
+
+	if (reached_maxtime())
+		spinning_interrupted = 1;
+
+	if (! Options.verbose)
+		return;
+
 	spin_state++;
 	if (spin_state >= sizeof patterns)
 		spin_state = 0;
@@ -177,6 +215,9 @@ static void spin_update(void)
 
 static void spin_suspend(void)
 {
+	if (! Options.verbose)
+		return;
+
 	printf("%s %s", cub2, cub2);
 	fflush(stdout);
 	if (spinning_interrupted)
@@ -223,18 +264,6 @@ static void initialize_signal_handlers(void)
 }
 
 
-typedef struct
-{
-	const char *pname;
-	Boolean analyse_atr;
-	Boolean stress_card;
-	Boolean print_version;
-	Boolean verbose;
-	Boolean only_list_readers;
-} options_t;
-
-static options_t Options;
-
 static void initialize_options(options_t *options, const char *pname)
 {
 	options->pname = pname;
@@ -245,14 +274,16 @@ static void initialize_options(options_t *options, const char *pname)
 #endif
 	options->stress_card = False;
 	options->print_version = False;
+	options->maxtime = 0;
 	options->verbose = True;
 	options->only_list_readers = False;
+	options->only_list_cards = False;
 }
 
 #ifdef WIN32
-#define OPTIONS "Vhvrsq"
+#define OPTIONS "Vhvrcst:q"
 #else
-#define OPTIONS "Vhnvrsq"
+#define OPTIONS "Vhnvrcst:q"
 #endif
 
 static int parse_options(int argc, char *argv[], options_t *options)
@@ -278,12 +309,23 @@ static int parse_options(int argc, char *argv[], options_t *options)
 
 			case 'r':
 				options->only_list_readers = True;
+				options->verbose = False;
+				break;
+
+			case 'c':
+				options->only_list_cards = True;
+				options->verbose = False;
+				options->analyse_atr = False;
 				break;
 
 			case 's':
 				options->stress_card = True;
 				break;
 
+			case 't':
+				options->maxtime = atol(optarg);
+				break;
+
 			case 'q':
 				options->verbose = False;
 				break;
@@ -430,6 +472,7 @@ int main(int argc, char *argv[])
 	char atr_command[sizeof(atr)+sizeof(ATR_PARSER)+2+1];
 	int pnp = TRUE;
 
+	start_time = time(NULL);
 	initialize_terminal();
 	if (0 != parse_options(argc, argv, &Options))
 	{
@@ -532,6 +575,12 @@ get_readers:
 
 	if (SCARD_E_NO_READERS_AVAILABLE == rv || 0 == nbReaders)
 	{
+		if (Options.only_list_cards || Options.only_list_readers)
+		{
+			printf("No reader found.\n");
+			return EX_OK;
+		}
+
 		if (Options.verbose)
 		{
 			printf("%sWaiting for the first reader...%s   ", red, color_end);
@@ -599,7 +648,8 @@ get_readers:
 		nbReaders++;
 	}
 
-	print_readers(readers, nbReaders);
+	if (! Options.only_list_cards)
+		print_readers(readers, nbReaders);
 	if (Options.only_list_readers)
 	{
 		exit(EX_OK);
@@ -817,6 +867,9 @@ get_readers:
 			}
 		} /* for */
 
+		if (Options.only_list_cards)
+			break;
+
 		rv = SCardGetStatusChange(hContext, TIMEOUT, rgReaderStates_t,
 			nbReaders);
 
